
#include <cstdio>
#include <vector>
#include <cassert>
#include <cstdlib>
#include <string>
#include <sstream>
#include <iostream>
#include <algorithm>
using namespace std;

struct Vertex {
	int value;
	int minValue;
	int maxValue;
	int subtreeSize;
	bool reversed;
	Vertex* leftChild;
	Vertex* rightChild;

	void reverse() {
		if (reversed) {
			Vertex* swapVertex = leftChild;
			leftChild = rightChild;
			rightChild = swapVertex;
			reversed = false;

			if (leftChild != NULL)
				leftChild->reversed = !leftChild->reversed;
			if (rightChild != NULL)
				rightChild->reversed = !rightChild->reversed;
		}
	}

	void fullReverse() {
		reverse();
		if (leftChild != NULL)
			leftChild->fullReverse();
		if (rightChild != NULL)
			rightChild->fullReverse();
	}

	void dfs(vector<int> &seq) {
		if (leftChild != NULL)
			leftChild->dfs(seq);
		seq.push_back(value);
		if (rightChild != NULL)
			rightChild->dfs(seq);
	}
};

inline Vertex* createTree(int value) {
	Vertex* root = new Vertex();
	root->value = value;
	root->minValue = value;
	root->maxValue = value;
	root->subtreeSize = 1;
	root->reversed = false;
	root->leftChild = NULL;
	root->rightChild = NULL;
	return root;
}

inline int getSizeOfTree(Vertex *root) {
	return (root == NULL) ? 0 : root->subtreeSize;
}

inline void update(Vertex *root) {
	if (root == NULL)
		return;
	root->minValue = root->value;
	root->maxValue = root->value;
	root->subtreeSize = 1;
	if (root->leftChild != NULL) {
		root->minValue = min(root->minValue, root->leftChild->minValue);
		root->maxValue = max(root->maxValue, root->leftChild->maxValue);
		root->subtreeSize += root->leftChild->subtreeSize;
	}
	if (root->rightChild != NULL) {
		root->minValue = min(root->minValue, root->rightChild->minValue);
		root->maxValue = max(root->maxValue, root->rightChild->maxValue);
		root->subtreeSize += root->rightChild->subtreeSize;
	}
}

Vertex* mergeTrees(Vertex* leftChild, Vertex* rightChild) {
	if (leftChild == NULL)
		return rightChild;
	if (rightChild == NULL)
		return leftChild;

	leftChild->reverse();
	rightChild->reverse();
	int leftSubtreeSize = getSizeOfTree(leftChild);
	int rightSubtreeSize = getSizeOfTree(rightChild);
	Vertex* root;
	if (rand() % (leftSubtreeSize + rightSubtreeSize) < leftSubtreeSize) {
		leftChild->rightChild = mergeTrees(leftChild->rightChild, rightChild);
		root = leftChild;
	} else {
		rightChild->leftChild = mergeTrees(leftChild, rightChild->leftChild);
		root = rightChild;
	}
	update(root);
	return root;
}

void splitTrees(Vertex **leftChild, Vertex **rightChild, Vertex *root, int splitIndex) {
	if (root == NULL) {
		*leftChild = NULL;
		*rightChild = NULL;
		return;
	}

	root->reverse();
	int rootIndex = getSizeOfTree(root->leftChild) + 1;

	if (rootIndex < splitIndex) {
		*leftChild = root;
		splitTrees(&((*leftChild)->rightChild), rightChild, root->rightChild, splitIndex - rootIndex);
	} else if (rootIndex > splitIndex) {
		*rightChild = root;
		splitTrees(leftChild, &((*rightChild)->leftChild), root->leftChild, splitIndex);
	} else {
		*leftChild = root;
		*rightChild = root->rightChild;
		root->rightChild = NULL;
	}
	update(*leftChild);
	update(*rightChild);
}

Vertex* buildTree(vector<int> &seq) {
	Vertex* root = NULL;
	for (int i = 0; i < int(seq.size()); ++i) {
		root = mergeTrees(root, createTree(seq[i]));
	}
	return root;
}

Vertex* applyReversal(Vertex* root, int first, int last) {
	Vertex *alpha, *beta, *gamma, *delta;
	splitTrees(&delta, &gamma, root, last);
	splitTrees(&alpha, &beta, delta, first - 1);
	beta->reversed = !beta->reversed;
	root = mergeTrees(alpha, beta);
	root = mergeTrees(root, gamma);
	return root;
}

vector<int> getSequence(Vertex* root) {
	root->fullReverse();
	vector<int> seq;
	root->dfs(seq);
	return seq;
}

void runSingleStressTest(vector<int> &permSrc, vector<pair<int, int> > &args) {
	Vertex *root = buildTree(permSrc);
	for (int i = 0; i < int(args.size()); ++i) {
		int first = min(args[i].first, args[i].second);
		int last = max(args[i].first, args[i].second);
//		printf("[first, last] = [%d, %d]\n", first, last);
		int length = last - first + 1;
		for (int j = 0; j < length / 2; ++j) {
			int swapValue = permSrc[first - 1 + j];
			permSrc[first - 1 + j] = permSrc[last - 1 - j];
			permSrc[last - 1 - j] = swapValue;
		}

//		for (int j = 0; j < permSrc.size(); ++j) printf("%d ", permSrc[j]); puts("");
		root = applyReversal(root, first, last);
//		vector<int> seq = getSequence(root);
//		for (int j = 0; j < permSrc.size(); ++j) printf("%d ", seq[j]); puts("");
		assert(getSequence(root) == permSrc);
	}
}

inline int randomInt(int a, int b) {
	return a + rand() % (b - a + 1);
}

void simpleTest() {
	int permSrcArray[] = {5, 2, 1, 4, 3};
	vector<int> permSrc(permSrcArray, permSrcArray + 5);
	vector<pair<int, int> > args;
	args.push_back(make_pair(1, 3));
	args.push_back(make_pair(2, 5));
	args.push_back(make_pair(2, 4));
	args.push_back(make_pair(1, 5));
	args.push_back(make_pair(3, 4));
	runSingleStressTest(permSrc, args);
}

void randomTest(int length) {
	vector<int> permSrc(length);
	for (int i = 0; i < length; ++i)
		permSrc[i] = i + 1;
	random_shuffle(permSrc.begin(), permSrc.end());
	vector<pair<int, int> > args;
	for (int i = 0; i < length; ++i) {
		int first = randomInt(1, length);
		int last = randomInt(first, length);
		if (rand() % 2)
			swap(first, last);
		args.push_back(make_pair(first, last));
	}
	runSingleStressTest(permSrc, args);
}

void stressTestDataStructures() {
	simpleTest();
	while (true) {
		int len = randomInt(1000, 5000);
		randomTest(len);
		cout << "." << flush;
	}
}

void readInput(FILE *fin, int &length, vector<int> &srcPerm, vector<int> &dstPerm) {
	fscanf(fin, "%d", &length);
	srcPerm = vector<int>(length, -1);
	dstPerm = vector<int>(length, -1);
	for (int i = 0; i < length; ++i) fscanf(fin, "%d", &srcPerm[i]);
	for (int i = 0; i < length; ++i) fscanf(fin, "%d", &dstPerm[i]);
}

void terminateCheckerWith(string message) {
	printf("%s", message.c_str());
	exit(0);
}

inline int nextInt(FILE *fin) {
	int res = 0;
	if (feof(fin) || fscanf(fin, "%d", &res) != 1) {
		terminateCheckerWith("0\nError: badly formatted contestant's output file.\n");
	}
	return res;
}

string convertToString(int num) {
	stringstream ss;
	ss << num;
	string res;
	ss >> res;
	return res;
}

void runChecker(int length, vector<int> &initialPerm, vector<int> &finalPerm,
	vector<pair<int, int> > &reverseArgs) {

	Vertex *root = buildTree(initialPerm);
	for (int i = 0; i < int(reverseArgs.size()); ++i) {
		int first = reverseArgs[i].first;
		int last = reverseArgs[i].second;
		assert(first <= last);
		assert(first >= 1 && first <= length);
		assert(last >= 1 && last <= length);
		root = applyReversal(root, first, last);
//		vector<int> debug = getSequence(root);
//		for (int j = 0; j < debug.size(); ++j) printf("%d%c", debug[j], j < debug.size() - 1 ? ' ' : '\n');
	}
//	printf("need\n");
//	for (int j = 0; j < finalPerm.size(); ++j) printf("%d%c", finalPerm[j], j < finalPerm.size() - 1 ? ' ' : '\n');
	if (getSequence(root) != finalPerm) {
		terminateCheckerWith("0\nWrong answer: the provided list of operations doesn't lead to the desired sequence!\n");
	} else {
		terminateCheckerWith("25\nCorrect solution with " + convertToString(reverseArgs.size()) + " operation(s)!\n");
	}
}

#warning IMPLEMENT HACKING CHECK IN THE BEGINNING TO IMPROVE SPEED ON WRONG SOLUTIONS BY REVERSING SINGLE ELEMENTS Z TIMES

int main(int argc, char *argv[]) {
	if (argc != 5) {
		terminateCheckerWith("0\nError: wrong number of arguments passed to checker!\n");
	}
	FILE *fin, *fout;
	if (!(fin = fopen(argv[1], "r"))) {
		terminateCheckerWith("0\nError: cannot open testcase input file!\n");
	}
	if (!(fout = fopen(argv[2], "r"))) {
		terminateCheckerWith("0\nError: cannot open contestant's output file!\n");
	}

	int length;
	vector<int> srcPerm;
	vector<int> dstPerm;
	readInput(fin, length, srcPerm, dstPerm);
	fclose(fin);

	int reversesCount = nextInt(fout);
	if (reversesCount < 0 || reversesCount > 1000000) {
		terminateCheckerWith("0\nError on line " + convertToString(1) +
			": the number of reverses must be between [0, 1000000], inclusive.\n");
	}

	vector<pair<int, int> > reverseArgs(reversesCount);
	for (int i = 0; i < reversesCount; ++i) {
		int first = nextInt(fout);
		int last = nextInt(fout);
		if (first > last)
			swap(first, last);
		if (first < 0 || first > length || last < 0 || last > length)
			terminateCheckerWith("0\nError on operation " + convertToString(i + 1) +
				": provided index is not between [1, " + convertToString(length) + "], inclusive.\n");
		reverseArgs[i] = make_pair(first, last);
	}
	fclose(fout);

	runChecker(length, srcPerm, dstPerm, reverseArgs);
	return 0;
}
